@@ -1,7 +1,7 @@
 /*
    ASCII strings utilities
 
-   Copyright (C) 2007-2017
+   Copyright (C) 2007-2018
    Free Software Foundation, Inc.
 
    Written by:
@@ -57,11 +57,11 @@ str_ascii_insert_replace_char (GString * buffer)
 
 /* --------------------------------------------------------------------------------------------- */
 
-static int
+static gboolean
 str_ascii_is_valid_string (const char *text)
 {
     (void) text;
-    return 1;
+    return TRUE;
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -116,7 +116,7 @@ str_ascii_cprev_noncomb_char (const char **text, const char *begin)
 
 /* --------------------------------------------------------------------------------------------- */
 
-static int
+static gboolean
 str_ascii_isspace (const char *text)
 {
     return g_ascii_isspace ((gchar) text[0]);
@@ -124,7 +124,7 @@ str_ascii_isspace (const char *text)
 
 /* --------------------------------------------------------------------------------------------- */
 
-static int
+static gboolean
 str_ascii_ispunct (const char *text)
 {
     return g_ascii_ispunct ((gchar) text[0]);
@@ -132,7 +132,7 @@ str_ascii_ispunct (const char *text)
 
 /* --------------------------------------------------------------------------------------------- */
 
-static int
+static gboolean
 str_ascii_isalnum (const char *text)
 {
     return g_ascii_isalnum ((gchar) text[0]);
@@ -140,7 +140,7 @@ str_ascii_isalnum (const char *text)
 
 /* --------------------------------------------------------------------------------------------- */
 
-static int
+static gboolean
 str_ascii_isdigit (const char *text)
 {
     return g_ascii_isdigit ((gchar) text[0]);
@@ -148,7 +148,7 @@ str_ascii_isdigit (const char *text)
 
 /* --------------------------------------------------------------------------------------------- */
 
-static int
+static gboolean
 str_ascii_isprint (const char *text)
 {
     return g_ascii_isprint ((gchar) text[0]);
@@ -169,26 +169,26 @@ static int
 str_ascii_toupper (const char *text, char **out, size_t * remain)
 {
     if (*remain <= 1)
-        return 0;
+        return FALSE;
 
     (*out)[0] = (char) g_ascii_toupper ((gchar) text[0]);
     (*out)++;
     (*remain)--;
-    return 1;
+    return TRUE;
 }
 
 /* --------------------------------------------------------------------------------------------- */
 
-static int
+static gboolean
 str_ascii_tolower (const char *text, char **out, size_t * remain)
 {
     if (*remain <= 1)
-        return 0;
+        return FALSE;
 
     (*out)[0] = (char) g_ascii_tolower ((gchar) text[0]);
     (*out)++;
     (*remain)--;
-    return 1;
+    return TRUE;
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -551,7 +551,7 @@ str_ascii_column_to_pos (const char *text, size_t pos)
 /* --------------------------------------------------------------------------------------------- */
 
 static char *
-str_ascii_create_search_needle (const char *needle, int case_sen)
+str_ascii_create_search_needle (const char *needle, gboolean case_sen)
 {
     (void) case_sen;
     return (char *) needle;
@@ -560,7 +560,7 @@ str_ascii_create_search_needle (const char *needle, int case_sen)
 /* --------------------------------------------------------------------------------------------- */
 
 static void
-str_ascii_release_search_needle (char *needle, int case_sen)
+str_ascii_release_search_needle (char *needle, gboolean case_sen)
 {
     (void) case_sen;
     (void) needle;
@@ -570,14 +570,14 @@ str_ascii_release_search_needle (char *needle, int case_sen)
 /* --------------------------------------------------------------------------------------------- */
 
 static const char *
-str_ascii_search_first (const char *text, const char *search, int case_sen)
+str_ascii_search_first (const char *text, const char *search, gboolean case_sen)
 {
     char *fold_text;
     char *fold_search;
     const char *match;
 
-    fold_text = (case_sen) ? (char *) text : g_ascii_strdown (text, -1);
-    fold_search = (case_sen) ? (char *) search : g_ascii_strdown (search, -1);
+    fold_text = case_sen ? (char *) text : g_ascii_strdown (text, -1);
+    fold_search = case_sen ? (char *) search : g_ascii_strdown (search, -1);
 
     match = g_strstr_len (fold_text, -1, fold_search);
     if (match != NULL)
@@ -600,14 +600,14 @@ str_ascii_search_first (const char *text, const char *search, int case_sen)
 /* --------------------------------------------------------------------------------------------- */
 
 static const char *
-str_ascii_search_last (const char *text, const char *search, int case_sen)
+str_ascii_search_last (const char *text, const char *search, gboolean case_sen)
 {
     char *fold_text;
     char *fold_search;
     const char *match;
 
-    fold_text = (case_sen) ? (char *) text : g_ascii_strdown (text, -1);
-    fold_search = (case_sen) ? (char *) search : g_ascii_strdown (search, -1);
+    fold_text = case_sen ? (char *) text : g_ascii_strdown (text, -1);
+    fold_search = case_sen ? (char *) search : g_ascii_strdown (search, -1);
 
     match = g_strrstr_len (fold_text, -1, fold_search);
     if (match != NULL)
@@ -671,7 +671,7 @@ str_ascii_fix_string (char *text)
 /* --------------------------------------------------------------------------------------------- */
 
 static char *
-str_ascii_create_key (const char *text, int case_sen)
+str_ascii_create_key (const char *text, gboolean case_sen)
 {
     (void) case_sen;
     return (char *) text;
@@ -680,15 +680,15 @@ str_ascii_create_key (const char *text, int case_sen)
 /* --------------------------------------------------------------------------------------------- */
 
 static int
-str_ascii_key_collate (const char *t1, const char *t2, int case_sen)
+str_ascii_key_collate (const char *t1, const char *t2, gboolean case_sen)
 {
-    return (case_sen) ? strcmp (t1, t2) : g_ascii_strcasecmp (t1, t2);
+    return case_sen ? strcmp (t1, t2) : g_ascii_strcasecmp (t1, t2);
 }
 
 /* --------------------------------------------------------------------------------------------- */
 
 static void
-str_ascii_release_key (char *key, int case_sen)
+str_ascii_release_key (char *key, gboolean case_sen)
 {
     (void) key;
     (void) case_sen;
