@@ -1,7 +1,7 @@
 /*
    Virtual File System switch code
 
-   Copyright (C) 1995-2017
+   Copyright (C) 1995-2018
    Free Software Foundation, Inc.
 
    Written by: 1995 Miguel de Icaza
@@ -45,6 +45,15 @@
 #include <errno.h>
 #include <stdlib.h>
 
+#ifdef __linux__
+#ifdef HAVE_LINUX_FS_H
+#include <linux/fs.h>
+#endif /* HAVE_LINUX_FS_H */
+#ifdef HAVE_SYS_IOCTL_H
+#include <sys/ioctl.h>
+#endif /* HAVE_SYS_IOCTL_H */
+#endif /* __linux__ */
+
 #include "lib/global.h"
 #include "lib/strutil.h"
 #include "lib/util.h"
@@ -73,10 +82,6 @@ vfs_class *current_vfs = NULL;
 
 /*** file scope macro definitions ****************************************************************/
 
-#if defined(_AIX) && !defined(NAME_MAX)
-#define NAME_MAX FILENAME_MAX
-#endif
-
 #define VFS_FIRST_HANDLE 100
 
 /*** file scope type declarations ****************************************************************/
@@ -641,9 +646,6 @@ vfs_preallocate (int dest_vfs_fd, off_t src_fsize, off_t dest_fsize)
     void *dest_fd = NULL;
     struct vfs_class *dest_class;
 
-    if (!mc_global.vfs.preallocate_space)
-        return 0;
-
     if (src_fsize == 0)
         return 0;
 
@@ -654,6 +656,50 @@ vfs_preallocate (int dest_vfs_fd, off_t src_fsize, off_t dest_fsize)
     return posix_fallocate (*(int *) dest_fd, dest_fsize, src_fsize - dest_fsize);
 
 #endif /* HAVE_POSIX_FALLOCATE */
+}
+
+ /* --------------------------------------------------------------------------------------------- */
+
+int
+vfs_clone_file (int dest_vfs_fd, int src_vfs_fd)
+{
+#ifdef FICLONE
+    void *dest_fd = NULL;
+    void *src_fd = NULL;
+    struct vfs_class *dest_class;
+    struct vfs_class *src_class;
+
+    dest_class = vfs_class_find_by_handle (dest_vfs_fd, &dest_fd);
+    if ((dest_class->flags & VFSF_LOCAL) == 0)
+    {
+        errno = EOPNOTSUPP;
+        return (-1);
+    }
+    if (dest_fd == NULL)
+    {
+        errno = EBADF;
+        return (-1);
+    }
+
+    src_class = vfs_class_find_by_handle (src_vfs_fd, &src_fd);
+    if ((src_class->flags & VFSF_LOCAL) == 0)
+    {
+        errno = EOPNOTSUPP;
+        return (-1);
+    }
+    if (src_fd == NULL)
+    {
+        errno = EBADF;
+        return (-1);
+    }
+
+    return ioctl (*(int *) dest_fd, FICLONE, *(int *) src_fd);
+#else
+    (void) dest_vfs_fd;
+    (void) src_vfs_fd;
+    errno = EOPNOTSUPP;
+    return (-1);
+#endif
 }
 
 /* --------------------------------------------------------------------------------------------- */
