@@ -1,7 +1,7 @@
 /*
    Dialog box features module for the Midnight Commander
 
-   Copyright (C) 1994-2014
+   Copyright (C) 1994-2018
    Free Software Foundation, Inc.
 
    This file is part of the Midnight Commander.
@@ -42,6 +42,7 @@
 #include "lib/strutil.h"
 #include "lib/fileloc.h"        /* MC_HISTORY_FILE */
 #include "lib/event.h"          /* mc_event_raise() */
+#include "lib/util.h"           /* MC_PTR_FREE */
 
 #include "lib/widget.h"
 #include "lib/widget/mouse.h"
@@ -62,10 +63,10 @@ hook_t *idle_hook = NULL;
 
 /* If set then dialogs just clean the screen when refreshing, else */
 /* they do a complete refresh, refreshing all the parts of the program */
-int fast_refresh = 0;
+gboolean fast_refresh = FALSE;
 
 /* left click outside of dialog closes it */
-int mouse_close_dialog = 0;
+gboolean mouse_close_dialog = FALSE;
 
 const global_keymap_t *dialog_map = NULL;
 
@@ -653,6 +654,7 @@ dlg_default_repaint (WDialog * h)
 
     if (h->title != NULL)
     {
+        /* TODO: truncate long title */
         tty_setcolor (h->color[DLG_COLOR_TITLE]);
         widget_move (h, space, (wh->cols - str_term_width1 (h->title)) / 2);
         tty_print_string (h->title);
@@ -783,16 +785,7 @@ dlg_create (gboolean modal, int y1, int x1, int lines, int cols, widget_pos_flag
 
     new_d->mouse_status = MOU_UNHANDLED;
 
-    /* Strip existing spaces, add one space before and after the title */
-    if (title != NULL && *title != '\0')
-    {
-        char *t;
-
-        t = g_strstrip (g_strdup (title));
-        if (*t != '\0')
-            new_d->title = g_strdup_printf (" %s ", t);
-        g_free (t);
-    }
+    dlg_set_title (new_d, title);
 
     /* unique name of event group for this dialog */
     new_d->event_group = g_strdup_printf ("%s_%p", MCEVENT_GROUP_DIALOG, (void *) new_d);
@@ -845,6 +838,7 @@ add_widget_autopos (WDialog * h, void *w, widget_pos_flags_t pos_flags, const vo
 {
     Widget *wh = WIDGET (h);
     Widget *widget;
+    GList *new_current;
 
     /* Don't accept 0 widgets */
     if (w == NULL)
@@ -867,7 +861,7 @@ add_widget_autopos (WDialog * h, void *w, widget_pos_flags_t pos_flags, const vo
     if (h->widgets == NULL || before == NULL)
     {
         h->widgets = g_list_append (h->widgets, widget);
-        h->current = g_list_last (h->widgets);
+        new_current = g_list_last (h->widgets);
     }
     else
     {
@@ -882,17 +876,19 @@ add_widget_autopos (WDialog * h, void *w, widget_pos_flags_t pos_flags, const vo
         b = g_list_next (b);
         h->widgets = g_list_insert_before (h->widgets, b, widget);
         if (b != NULL)
-            h->current = g_list_previous (b);
+            new_current = g_list_previous (b);
         else
-            h->current = g_list_last (h->widgets);
+            new_current = g_list_last (h->widgets);
     }
 
-    /* widget has been added in runtime */
+    /* widget has been added at runtime */
     if (widget_get_state (wh, WST_ACTIVE))
     {
         send_message (widget, NULL, MSG_INIT, 0, NULL);
-        widget_set_state (widget, WST_FOCUSED, TRUE);
+        widget_select (widget);
     }
+    else
+        h->current = new_current;
 
     return widget->id;
 }
@@ -1143,7 +1139,8 @@ dlg_init (WDialog * h)
     widget_set_state (wh, WST_ACTIVE, TRUE);
     dlg_redraw (h);
     /* focus found widget */
-    widget_set_state (WIDGET (h->current->data), WST_FOCUSED, TRUE);
+    if (h->current != NULL)
+        widget_set_state (WIDGET (h->current->data), WST_FOCUSED, TRUE);
 
     h->ret_value = 0;
 }
@@ -1175,7 +1172,7 @@ dlg_run_done (WDialog * h)
 
     if (widget_get_state (WIDGET (h), WST_CLOSED))
     {
-        send_message (h, h->current->data, MSG_END, 0, NULL);
+        send_message (h, h->current == NULL ? NULL : WIDGET (h->current->data), MSG_END, 0, NULL);
         if (!widget_get_state (WIDGET (h), WST_MODAL))
             dialog_switch_remove (h);
     }
@@ -1254,6 +1251,25 @@ dlg_save_history (WDialog * h)
 
 /* --------------------------------------------------------------------------------------------- */
 
+void
+dlg_set_title (WDialog * h, const char *title)
+{
+    MC_PTR_FREE (h->title);
+
+    /* Strip existing spaces, add one space before and after the title */
+    if (title != NULL && title[0] != '\0')
+    {
+        char *t;
+
+        t = g_strstrip (g_strdup (title));
+        if (t[0] != '\0')
+            h->title = g_strdup_printf (" %s ", t);
+        g_free (t);
+    }
+}
+
+/* --------------------------------------------------------------------------------------------- */
+
 char *
 dlg_get_title (const WDialog * h, size_t len)
 {
