@@ -6,12 +6,15 @@
 action=$1
 filetype=$2
 
+[ -n "${MC_XDG_OPEN}" ] || MC_XDG_OPEN="xdg-open"
+
 do_view_action() {
     filetype=$1
 
     case "${filetype}" in
     jpeg)
-        identify "${MC_EXT_FILENAME}"; test -x /usr/bin/exif && echo && exif "${MC_EXT_FILENAME}" 2>/dev/null
+        identify "${MC_EXT_FILENAME}"
+        which exif >/dev/null 2>&1 && exif "${MC_EXT_FILENAME}" 2>/dev/null
         ;;
     xpm)
         sxpm "${MC_EXT_FILENAME}"
@@ -27,16 +30,21 @@ do_open_action() {
 
     case "${filetype}" in
     xbm)
-        bitmap "${MC_EXT_FILENAME}"
+        (bitmap "${MC_EXT_FILENAME}" &)
         ;;
     xcf)
         (gimp "${MC_EXT_FILENAME}" &)
         ;;
+    svg)
+        (inkscape "${MC_EXT_FILENAME}" &)
+        ;;
     *)
-        if [ "$DISPLAY" = "" ]; then
-            zgv "${MC_EXT_FILENAME}"
-        else
+        if [ -n "$DISPLAY" ]; then
             (gqview "${MC_EXT_FILENAME}" &)
+        elif see >/dev/null 2>&1; then
+            (see "${MC_EXT_FILENAME}" &)
+        else
+            (zgv "${MC_EXT_FILENAME}" &)
         fi
         ;;
     esac
@@ -47,7 +55,7 @@ view)
     do_view_action "${filetype}"
     ;;
 open)
-    xdg-open "${MC_EXT_FILENAME}" 2>/dev/null || \
+    ("${MC_XDG_OPEN}" "${MC_EXT_FILENAME}" >/dev/null 2>&1) || \
         do_open_action "${filetype}"
     ;;
 *)
