@@ -1,7 +1,7 @@
 /*
    Search functions for diffviewer.
 
-   Copyright (C) 2010-2016
+   Copyright (C) 2010-2018
    Free Software Foundation, Inc.
 
    Written by:
@@ -74,7 +74,7 @@ mcdiffview_dialog_search (WDiff * dview)
 {
     char *exp = NULL;
     int qd_result;
-    size_t num_of_types;
+    size_t num_of_types = 0;
     gchar **list_of_types;
 
     list_of_types = mc_search_get_types_strings_array (&num_of_types);
@@ -123,11 +123,8 @@ mcdiffview_dialog_search (WDiff * dview)
         GString *tmp;
 
         tmp = str_convert_to_input (exp);
-        if (tmp != NULL)
-        {
-            g_free (exp);
-            exp = g_string_free (tmp, FALSE);
-        }
+        g_free (exp);
+        exp = g_string_free (tmp, FALSE);
     }
 #endif
 
