@@ -1,7 +1,7 @@
 /*
    Chown command -- for the Midnight Commander
 
-   Copyright (C) 1994-2017
+   Copyright (C) 1994-2018
    Free Software Foundation, Inc.
 
    This file is part of the Midnight Commander.
@@ -45,9 +45,6 @@
 #include "lib/widget.h"
 
 #include "src/setup.h"          /* panels_options */
-
-/* Needed for the extern declarations of integer parameters */
-#include "chmod.h"
 #include "midnight.h"           /* current_panel */
 
 #include "chown.h"
@@ -73,40 +70,47 @@
 
 /*** file scope variables ************************************************************************/
 
-static int need_update, end_chown;
-static int current_file;
-static int single_set;
-static WListbox *l_user, *l_group;
 
-/* *INDENT-OFF* */
 static struct
 {
     int ret_cmd;
     button_flags_t flags;
-    int y, len;
+    int y;
+    int len;
     const char *text;
-} chown_but[BUTTONS] = {
-    { B_SETALL, NORMAL_BUTTON,  5, 0, N_("Set &all") },
-    { B_SETGRP, NORMAL_BUTTON,  5, 0, N_("Set &groups") },
-    { B_SETUSR, NORMAL_BUTTON,  5, 0, N_("Set &users") },
-    { B_ENTER,  DEFPUSH_BUTTON, 3, 0, N_("&Set") },
-    { B_CANCEL, NORMAL_BUTTON,  3, 0, N_("&Cancel") },
+} chown_but[BUTTONS] =
+{
+    /* *INDENT-OFF* */
+    { B_SETALL,  NORMAL_BUTTON, 5, 0, N_("Set &all")    },
+    { B_SETGRP,  NORMAL_BUTTON, 5, 0, N_("Set &groups") },
+    { B_SETUSR,  NORMAL_BUTTON, 5, 0, N_("Set &users")  },
+    { B_ENTER,  DEFPUSH_BUTTON, 3, 0, N_("&Set")        },
+    { B_CANCEL,  NORMAL_BUTTON, 3, 0, N_("&Cancel")     }
+    /* *INDENT-ON* */
 };
 
 /* summary length of three buttons */
 static int blen = 0;
 
-static struct {
+static struct
+{
     int y;
     WLabel *l;
-} chown_label [LABELS] = {
-    { 4 , NULL },
-    { 6 , NULL },
-    { 8 , NULL },
-    { 10 , NULL },
+} chown_label[LABELS] =
+{
+    /* *INDENT-OFF* */
+    {  4, NULL },
+    {  6, NULL },
+    {  8, NULL },
+    { 10, NULL },
     { 12, NULL }
+    /* *INDENT-ON* */
 };
-/* *INDENT-ON* */
+
+static int current_file;
+static gboolean ignore_all;
+
+static WListbox *l_user, *l_group;
 
 /* --------------------------------------------------------------------------------------------- */
 /*** file scope functions ************************************************************************/
@@ -116,7 +120,7 @@ static void
 chown_i18n (void)
 {
     static gboolean i18n = FALSE;
-    unsigned int i;
+    int i;
 
     if (i18n)
         return;
@@ -146,8 +150,8 @@ chown_i18n (void)
 static void
 chown_refresh (WDialog * h)
 {
-    const int y = 3;
-    const int x = 7 + GW * 2;
+    int y = 3;
+    int x = 7 + GW * 2;
 
     dlg_default_repaint (h);
 
@@ -167,17 +171,6 @@ chown_refresh (WDialog * h)
 
 /* --------------------------------------------------------------------------------------------- */
 
-static char *
-next_file (void)
-{
-    while (!current_panel->dir.list[current_file].f.marked)
-        current_file++;
-
-    return current_panel->dir.list[current_file].fname;
-}
-
-/* --------------------------------------------------------------------------------------------- */
-
 static cb_ret_t
 chown_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *data)
 {
@@ -195,22 +188,18 @@ chown_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *d
 /* --------------------------------------------------------------------------------------------- */
 
 static WDialog *
-init_chown (void)
+chown_init (void)
 {
+    int single_set;
+    WDialog *ch_dlg;
     int lines, cols;
-    int i;
-    int y;
+    int i, y;
     struct passwd *l_pass;
     struct group *l_grp;
-    WDialog *ch_dlg;
 
-    do_refresh ();
-
-    end_chown = need_update = current_file = 0;
     single_set = (current_panel->marked < 2) ? 3 : 0;
-
+    lines = GH + 4 + (single_set != 0 ? 2 : 4);
     cols = GW * 3 + 2 + 6;
-    lines = GH + 4 + (single_set ? 2 : 4);
 
     ch_dlg =
         dlg_create (TRUE, 0, 0, lines, cols, WPOS_CENTER, FALSE, dialog_colors, chown_callback,
@@ -246,7 +235,7 @@ init_chown (void)
         add_widget (ch_dlg, chown_label[i].l);
     }
 
-    if (!single_set)
+    if (single_set == 0)
     {
         int x;
 
@@ -284,7 +273,7 @@ init_chown (void)
 /* --------------------------------------------------------------------------------------------- */
 
 static void
-chown_done (void)
+chown_done (gboolean need_update)
 {
     if (need_update)
         update_panels (UP_OPTIMIZE, UP_KEEPSEL);
@@ -293,35 +282,107 @@ chown_done (void)
 
 /* --------------------------------------------------------------------------------------------- */
 
-static void
-do_chown (uid_t u, gid_t g)
+static const char *
+next_file (void)
 {
-    vfs_path_t *vpath;
+    while (!current_panel->dir.list[current_file].f.marked)
+        current_file++;
 
-    vpath = vfs_path_from_str (current_panel->dir.list[current_file].fname);
-    if (mc_chown (vpath, u, g) == -1)
-        message (D_ERROR, MSG_ERROR, _("Cannot chown \"%s\"\n%s"),
-                 current_panel->dir.list[current_file].fname, unix_error_string (errno));
+    return current_panel->dir.list[current_file].fname;
+}
+
+/* --------------------------------------------------------------------------------------------- */
+
+static gboolean
+try_chown (const vfs_path_t * p, uid_t u, gid_t g)
+{
+    while (mc_chown (p, u, g) == -1 && !ignore_all)
+    {
+        int my_errno = errno;
+        int result;
+        char *msg;
+
+        msg =
+            g_strdup_printf (_("Cannot chown \"%s\"\n%s"), x_basename (vfs_path_as_str (p)),
+                             unix_error_string (my_errno));
+        result =
+            query_dialog (MSG_ERROR, msg, D_ERROR, 4, _("&Ignore"), _("Ignore &all"), _("&Retry"),
+                          _("&Cancel"));
+        g_free (msg);
+
+        switch (result)
+        {
+        case 0:
+            /* try next file */
+            return TRUE;
+
+        case 1:
+            ignore_all = TRUE;
+            /* try next file */
+            return TRUE;
+
+        case 2:
+            /* retry this file */
+            break;
+
+        case 3:
+        default:
+            /* stop remain files processing */
+            return FALSE;
+        }
+    }
+
+    return TRUE;
+}
+
+/* --------------------------------------------------------------------------------------------- */
+
+static gboolean
+do_chown (const vfs_path_t * p, uid_t u, gid_t g)
+{
+    gboolean ret;
+
+    ret = try_chown (p, u, g);
 
-    vfs_path_free (vpath);
     do_file_mark (current_panel, current_file, 0);
+
+    return ret;
 }
 
 /* --------------------------------------------------------------------------------------------- */
 
 static void
-apply_chowns (uid_t u, gid_t g)
+apply_chowns (vfs_path_t * vpath, uid_t u, gid_t g)
 {
+    gboolean ok;
 
-    need_update = end_chown = 1;
-    do_chown (u, g);
+    if (!do_chown (vpath, u, g))
+        return;
 
     do
     {
-        next_file ();
-        do_chown (u, g);
+        const char *fname;
+        struct stat sf;
+
+        fname = next_file ();
+        vpath = vfs_path_from_str (fname);
+        ok = (mc_stat (vpath, &sf) == 0);
+
+        if (!ok)
+        {
+            /* if current file was deleted outside mc -- try next file */
+            /* decrease current_panel->marked */
+            do_file_mark (current_panel, current_file, 0);
+
+            /* try next file */
+            ok = TRUE;
+        }
+        else
+            ok = do_chown (vpath, u, g);
+
+        vfs_path_free (vpath);
     }
-    while (current_panel->marked);
+    while (ok && current_panel->marked != 0);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -331,35 +392,44 @@ apply_chowns (uid_t u, gid_t g)
 void
 chown_cmd (void)
 {
-    char *fname;
-    struct stat sf_stat;
-    uid_t new_user;
-    gid_t new_group;
-    char buffer[BUF_TINY];
+    gboolean need_update;
+    gboolean end_chown;
 
     chown_i18n ();
 
+    current_file = 0;
+    ignore_all = FALSE;
+
     do
     {                           /* do while any files remaining */
         vfs_path_t *vpath;
         WDialog *ch_dlg;
+        struct stat sf_stat;
+        const char *fname;
+        int result;
+        char buffer[BUF_TINY];
+        uid_t new_user = (uid_t) (-1);
+        gid_t new_group = (gid_t) (-1);
 
-        ch_dlg = init_chown ();
-        new_user = new_group = -1;
+        do_refresh ();
 
-        if (current_panel->marked)
+        need_update = FALSE;
+        end_chown = FALSE;
+
+        if (current_panel->marked != 0)
             fname = next_file ();       /* next marked file */
         else
             fname = selection (current_panel)->fname;   /* single file */
 
         vpath = vfs_path_from_str (fname);
+
         if (mc_stat (vpath, &sf_stat) != 0)
-        {                       /* get status of file */
-            dlg_destroy (ch_dlg);
+        {
             vfs_path_free (vpath);
             break;
         }
-        vfs_path_free (vpath);
+
+        ch_dlg = chown_init ();
 
         /* select in listboxes */
         listbox_select_entry (l_user, listbox_search_text (l_user, get_owner (sf_stat.st_uid)));
@@ -372,12 +442,56 @@ chown_cmd (void)
         chown_label (3, buffer);
         chown_label (4, string_perm (sf_stat.st_mode));
 
-        switch (dlg_run (ch_dlg))
+        result = dlg_run (ch_dlg);
+
+        switch (result)
         {
         case B_CANCEL:
-            end_chown = 1;
+            end_chown = TRUE;
             break;
 
+        case B_ENTER:
+        case B_SETALL:
+            {
+                struct group *grp;
+                struct passwd *user;
+                char *text;
+
+                listbox_get_current (l_group, &text, NULL);
+                grp = getgrnam (text);
+                if (grp != NULL)
+                    new_group = grp->gr_gid;
+                listbox_get_current (l_user, &text, NULL);
+                user = getpwnam (text);
+                if (user != NULL)
+                    new_user = user->pw_uid;
+                if (result == B_ENTER)
+                {
+                    if (current_panel->marked <= 1)
+                    {
+                        /* single or last file */
+                        if (mc_chown (vpath, new_user, new_group) == -1)
+                            message (D_ERROR, MSG_ERROR, _("Cannot chown \"%s\"\n%s"),
+                                     fname, unix_error_string (errno));
+                        end_chown = TRUE;
+                    }
+                    else if (!try_chown (vpath, new_user, new_group))
+                    {
+                        /* stop multiple files processing */
+                        result = B_CANCEL;
+                        end_chown = TRUE;
+                    }
+                }
+                else
+                {
+                    apply_chowns (vpath, new_user, new_group);
+                    end_chown = TRUE;
+                }
+
+                need_update = TRUE;
+                break;
+            }
+
         case B_SETUSR:
             {
                 struct passwd *user;
@@ -385,10 +499,12 @@ chown_cmd (void)
 
                 listbox_get_current (l_user, &text, NULL);
                 user = getpwnam (text);
-                if (user)
+                if (user != NULL)
                 {
                     new_user = user->pw_uid;
-                    apply_chowns (new_user, new_group);
+                    apply_chowns (vpath, new_user, new_group);
+                    need_update = TRUE;
+                    end_chown = TRUE;
                 }
                 break;
             }
@@ -400,60 +516,33 @@ chown_cmd (void)
 
                 listbox_get_current (l_group, &text, NULL);
                 grp = getgrnam (text);
-                if (grp)
+                if (grp != NULL)
                 {
                     new_group = grp->gr_gid;
-                    apply_chowns (new_user, new_group);
+                    apply_chowns (vpath, new_user, new_group);
+                    need_update = TRUE;
+                    end_chown = TRUE;
                 }
                 break;
             }
 
-        case B_SETALL:
-        case B_ENTER:
-            {
-                struct group *grp;
-                struct passwd *user;
-                char *text;
-
-                listbox_get_current (l_group, &text, NULL);
-                grp = getgrnam (text);
-                if (grp)
-                    new_group = grp->gr_gid;
-                listbox_get_current (l_user, &text, NULL);
-                user = getpwnam (text);
-                if (user)
-                    new_user = user->pw_uid;
-                if (ch_dlg->ret_value == B_ENTER)
-                {
-                    vfs_path_t *fname_vpath;
-
-                    fname_vpath = vfs_path_from_str (fname);
-                    need_update = 1;
-                    if (mc_chown (fname_vpath, new_user, new_group) == -1)
-                        message (D_ERROR, MSG_ERROR, _("Cannot chown \"%s\"\n%s"),
-                                 fname, unix_error_string (errno));
-                    vfs_path_free (fname_vpath);
-                }
-                else
-                    apply_chowns (new_user, new_group);
-                break;
-            }
-
         default:
             break;
-        }                       /* switch */
+        }
 
-        if (current_panel->marked && ch_dlg->ret_value != B_CANCEL)
+        if (current_panel->marked != 0 && result != B_CANCEL)
         {
             do_file_mark (current_panel, current_file, 0);
-            need_update = 1;
+            need_update = TRUE;
         }
 
+        vfs_path_free (vpath);
+
         dlg_destroy (ch_dlg);
     }
-    while (current_panel->marked && !end_chown);
+    while (current_panel->marked != 0 && !end_chown);
 
-    chown_done ();
+    chown_done (need_update);
 }
 
 /* --------------------------------------------------------------------------------------------- */
