@@ -86,11 +86,10 @@ void save_setup_cmd (void);
 void user_file_menu_cmd (void);
 void info_cmd (void);
 void listing_cmd (void);
-void change_listing_cmd (void);
+void setup_listing_format_cmd (void);
 void quick_cmd_no_menu (void);
 void info_cmd_no_menu (void);
 void quick_view_cmd (void);
-void toggle_listing_cmd (void);
 #ifdef HAVE_CHARSET
 void encoding_cmd (void);
 #endif
