@@ -1801,7 +1801,7 @@ do_nc (void)
         /* don't handle VFS timestamps for dirs opened in panels */
         mc_event_destroy (MCEVENT_GROUP_CORE, "vfs_timestamp");
 
-        dir_list_clean (&panelized_panel.list);
+        dir_list_free_list (&panelized_panel.list);
     }
 
     /* Program end */
