@@ -6,7 +6,7 @@
    created and destroyed.  This is required for the future vfs layer,
    it will be possible to have tree views over virtual file systems.
 
-   Copyright (C) 1994-2017
+   Copyright (C) 1994-2018
    Free Software Foundation, Inc.
 
    Written by:
@@ -770,7 +770,7 @@ tree_move (WTree * tree, const char *default_dest)
 
     if (!S_ISDIR (buf.st_mode))
     {
-        file_error (_("Destination \"%s\" must be a directory\n%s"), dest);
+        file_error (TRUE, _("Destination \"%s\" must be a directory\n%s"), dest);
         goto ret;
     }
 
