@@ -1,7 +1,7 @@
 /* Virtual File System: SFTP file system.
    The VFS class functions
 
-   Copyright (C) 2011-2017
+   Copyright (C) 2011-2018
    Free Software Foundation, Inc.
 
    Written by:
@@ -150,7 +150,7 @@ sftpfs_cb_open (const vfs_path_t * vpath, int flags, mode_t mode)
     file_handler->ino = path_inode;
     file_handler->handle = -1;
     file_handler->changed = is_changed;
-    file_handler->linear = 0;
+    file_handler->linear = LS_NOT_LINEAR;
     file_handler->data = NULL;
 
     if (!sftpfs_open_file (file_handler, flags, mode, &mcerror))
@@ -324,18 +324,27 @@ sftpfs_cb_readlink (const vfs_path_t * vpath, char *buf, size_t size)
 /**
  * Callback for utime VFS-function.
  *
- * @param vpath unused
- * @param times unused
- * @return always 0
+ * @param vpath path to file or directory
+ * @param times access and modification time to set
+ * @return 0 if success, negative value otherwise
  */
 
 static int
 sftpfs_cb_utime (const vfs_path_t * vpath, mc_timesbuf_t * times)
 {
-    (void) vpath;
-    (void) times;
+    int rc;
+    GError *mcerror = NULL;
+#ifdef HAVE_UTIMENSAT
+    time_t atime = (*times)[0].tv_sec;
+    time_t mtime = (*times)[1].tv_sec;
+#else
+    time_t atime = times->actime;
+    time_t mtime = times->modtime;
+#endif
 
-    return 0;
+    rc = sftpfs_utime (vpath, atime, mtime, &mcerror);
+    mc_error_message (&mcerror, NULL);
+    return rc;
 }
 
 /* --------------------------------------------------------------------------------------------- */
